@@ -74,7 +74,7 @@ compile setup = do
     (output, s0) <-                             -- compile initial graph
         Prim.compile (runReaderT setup eventNetwork) Prim.emptyNetwork
     putMVar s s0                                -- set initial state
-        
+
     return $ eventNetwork
 
 fromAddHandler :: AddHandler a -> Moment (Event a)
@@ -112,22 +112,37 @@ imposeChanges = liftCached2 $ \(l1,_) p2 -> return (l1,p2)
 {-----------------------------------------------------------------------------
     Combinators - basic
 ------------------------------------------------------------------------------}
-never       = don'tCache  $ liftBuild $ Prim.neverP
+never :: Event a
+never = don'tCache  $ liftBuild $ Prim.neverP
+
+unionWith :: (a -> a -> a) -> Event a -> Event a -> Event a
 unionWith f = liftCached2 $ (liftBuild .) . Prim.unionWithP f
+
+filterJust :: Event (Maybe a) -> Event a
 filterJust  = liftCached1 $ liftBuild . Prim.filterJustP
-mapE f      = liftCached1 $ liftBuild . Prim.mapP f
-applyE      = liftCached2 $ \(~(lf,_)) px -> liftBuild $ Prim.applyP lf px
 
-changesB    = liftCached1 $ \(~(lx,px)) -> liftBuild $ Prim.tagFuture lx px
+mapE :: (a -> b) -> Event a -> Event b
+mapE f = liftCached1 $ liftBuild . Prim.mapP f
 
+applyE :: Behavior (a -> b) -> Event a -> Event b
+applyE = liftCached2 $ \(~(lf,_)) px -> liftBuild $ Prim.applyP lf px
+
+changesB :: Behavior a -> Event (Future a)
+changesB = liftCached1 $ \(~(lx,px)) -> liftBuild $ Prim.tagFuture lx px
+
+pureB :: a -> Behavior a
 pureB a = cache $ do
     p <- runCached never
     return (Prim.pureL a, p)
-applyB  = liftCached2 $ \(~(l1,p1)) (~(l2,p2)) -> liftBuild $ do
+
+applyB :: Behavior (a -> b) -> Behavior a -> Behavior b
+applyB = liftCached2 $ \(~(l1,p1)) (~(l2,p2)) -> liftBuild $ do
     p3 <- Prim.unionWithP const p1 p2
     let l3 = Prim.applyL l1 l2
     return (l3,p3)
-mapB f  = applyB (pureB f)
+
+mapB :: (a -> b) -> Behavior a -> Behavior b
+mapB f = applyB (pureB f)
 
 {-----------------------------------------------------------------------------
     Combinators - accumulation
@@ -147,6 +162,7 @@ cacheAndSchedule m = ask >>= \r -> liftBuild $ do
     Prim.buildLater $ void $ runReaderT (runCached c) r
     return c
 
+stepperB :: a -> Event a -> Moment (Behavior a)
 stepperB a e = cacheAndSchedule $ do
     p0 <- runCached e
     liftBuild $ do
@@ -155,6 +171,7 @@ stepperB a e = cacheAndSchedule $ do
         (l,_) <- Prim.accumL a p1
         return (l,p2)
 
+accumE :: a -> Event (a -> a) -> Moment (Event a)
 accumE a e1 = cacheAndSchedule $ do
     p0 <- runCached e1
     liftBuild $ do
@@ -184,7 +201,7 @@ executeP p1 = do
         p2 <- Prim.mapP runReaderT p1
         Prim.executeP p2 r
 
-observeE :: Event (Moment a) -> Event a 
+observeE :: Event (Moment a) -> Event a
 observeE = liftCached1 $ executeP
 
 executeE :: Event (Moment a) -> Moment (Event a)
@@ -217,4 +234,5 @@ switchB b e = ask >>= \r -> cacheAndSchedule $ do
         pr <- merge c1 =<< merge c2 c3
         return (lr, pr)
 
+merge :: Pulse () -> Pulse () -> Build (Pulse ())
 merge = Prim.unionWithP (\_ _ -> ())
